﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using ScheduleSample.Models;

namespace ScheduleSample.Controllers
{
    public class HomeController : Controller
    {
        ScheduleDataDataContext db = new ScheduleDataDataContext();
        public ActionResult Index()
        {
            return View();
        }  

        public JsonResult Load()
        {
            List<AppData> apps = new List<AppData>
            {
                new AppData{id = 1, subject = "test", type ="plan", StartTime = new DateTime(2022, 1, 1), EndTime = new DateTime(2022, 1, 2)}
            };
            var data = apps.ToList();
            return Json(data, JsonRequestBehavior.AllowGet);
        }
        
        public class AppData
        {
            public int id { get; set; }
            public string type { get; set; }
            public DateTime StartTime { get; set; }
            public DateTime EndTime { get; set; }
            public string subject { get; set; }

        }
        public JsonResult LoadData(Params param)  // Here we get the Id value we added in the dataQuery
        {
            var data = db.ScheduleEventDatas.ToList();
            return Json(data, JsonRequestBehavior.AllowGet);
        }

        public class Params
        {
            public DateTime StartDate { get; set; }
            public DateTime EndDate { get; set; }
        }

        public JsonResult LoadResources()
        {
            var data = db.ScheduleResourceDatas.ToList();
            return Json(data, JsonRequestBehavior.AllowGet);
        }

        [HttpPost]
        public JsonResult UpdateData(EditParams param)
        {
            if (param.action == "insert" || (param.action == "batch" && param.added != null)) // this block of code will execute while inserting the appointments
            {
                var value = (param.action == "insert") ? param.value : param.added[0];
                int intMax = db.ScheduleEventDatas.ToList().Count > 0 ? db.ScheduleEventDatas.ToList().Max(p => p.Id) : 1;
                DateTime startTime = Convert.ToDateTime(value.StartTime).ToLocalTime();
                DateTime endTime = Convert.ToDateTime(value.EndTime).ToLocalTime();

                List<ScheduleEventData> appointments = new List<ScheduleEventData>();

                for (int i = 0; i < param.added.Count; i++)
                {
                    appointments.Add(new ScheduleEventData()
                    {
                        Id = intMax + i + 1,
                        StartTime = startTime,
                        EndTime = endTime,
                        Subject = param.added[i].Subject,
                        Location = param.added[i].Location,
                        Description = param.added[i].Description,
                        IsAllDay = param.added[i].IsAllDay,
                        StartTimezone = param.added[i].StartTimezone,
                        EndTimezone = param.added[i].EndTimezone,
                        RecurrenceRule = param.added[i].RecurrenceRule,
                        RecurrenceID = param.added[i].RecurrenceID,
                        RecurrenceException = param.added[i].RecurrenceException,
                        ProjectId = param.added[i].ProjectId,
                        TaskId = param.added[i].TaskId

                    });
                    db.ScheduleEventDatas.InsertOnSubmit(appointments[i]);
                    db.SubmitChanges();
                }
            }
            if (param.action == "update" || (param.action == "batch" && param.changed != null)) // this block of code will execute while updating the appointment
            {
                DateTime start = param.Start;
                var value = (param.action == "update") ? param.value : param.changed[0];
                var filterData = db.ScheduleEventDatas.Where(c => c.Id == Convert.ToInt32(value.Id));
                if (filterData.Count() > 0)
                {
                    DateTime startTime = Convert.ToDateTime(value.StartTime);
                    DateTime endTime = Convert.ToDateTime(value.EndTime);
                    ScheduleEventData appointment = db.ScheduleEventDatas.Single(A => A.Id == Convert.ToInt32(value.Id));
                    appointment.StartTime = startTime;
                    appointment.EndTime = endTime;
                    appointment.StartTimezone = value.StartTimezone;
                    appointment.EndTimezone = value.EndTimezone;
                    appointment.Subject = value.Subject;
                    appointment.Location = value.Location;
                    appointment.Description = value.Description;
                    appointment.IsAllDay = value.IsAllDay;
                    appointment.RecurrenceRule = value.RecurrenceRule;
                    appointment.RecurrenceID = value.RecurrenceID;
                    appointment.RecurrenceException = value.RecurrenceException;
                    appointment.ProjectId = value.ProjectId;
                    appointment.TaskId = value.TaskId;
                }
                db.SubmitChanges();
            }
            if (param.action == "remove" || (param.action == "batch" && param.deleted != null)) // this block of code will execute while removing the appointment
            {
                if (param.action == "remove")
                {
                    int key = Convert.ToInt32(param.key);
                    ScheduleEventData appointment = db.ScheduleEventDatas.Where(c => c.Id == key).FirstOrDefault();
                    if (appointment != null) db.ScheduleEventDatas.DeleteOnSubmit(appointment);
                }
                else
                {
                    foreach (var apps in param.deleted)
                    {
                        ScheduleEventData appointment = db.ScheduleEventDatas.Where(c => c.Id == apps.Id).FirstOrDefault();
                        if (apps != null) db.ScheduleEventDatas.DeleteOnSubmit(appointment);
                    }
                }
                db.SubmitChanges();
            }
            var data = db.ScheduleEventDatas.ToList();
            return Json(data, JsonRequestBehavior.AllowGet);
        }

        public class EditParams
        {
            public string key { get; set; }
            public string action { get; set; }
            public List<ScheduleEventData> added { get; set; }
            public List<ScheduleEventData> changed { get; set; }
            public List<ScheduleEventData> deleted { get; set; }
            public ScheduleEventData value { get; set; }
            public DateTime Start { get; set; }
        }

        public ActionResult About()
        {
            ViewBag.Message = "Your application description page.";

            return View();
        }

        public ActionResult Contact()
        {
            ViewBag.Message = "Your contact page.";

            return View();
        }
    }
}